local addonName, addonTable = ...

local myRealm = GetRealmName()
local myName = UnitName("player")

local skills = addonTable.skills

local addon = addonTable.frame
addon.name = "Book of Crafts"
addon.menu = {}
InterfaceOptions_AddCategory(addon)

local title = addon:CreateFontString(nil, nil, "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetPoint("RIGHT", -16, 0)
title:SetJustifyH("LEFT")
title:SetJustifyV("TOP")
title:SetText("Book of Crafts")


local function onClick(self)
	SetItemRef(self.value:match("|H(.-)|h"), self.value)
end

local dropdown = CreateFrame("Frame")
dropdown.displayMode = "MENU"
dropdown.initialize = function(self, level)
	if level == 1 then
		for i, v in ipairs(addon.menu[myRealm].charNames) do
			if v ~= myName and #addon.menu[myRealm].data[v] > 0 then
				local info = UIDropDownMenu_CreateInfo()
				info.text = v
				info.hasArrow = true
				info.notCheckable = true
				UIDropDownMenu_AddButton(info, level)
			end
		end
	elseif level == 2 then
		for k, v in pairs(addon.realm[UIDROPDOWNMENU_MENU_VALUE].skills) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = skills[k]
			info.value = v.link
			info.func = onClick
			info.notCheckable = true
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

local dataobj = LibStub("LibDataBroker-1.1"):NewDataObject(addonName, {
	type = "launcher",
	icon = [[Interface\Icons\INV_Misc_Book_09]],
	label = "Book of Crafts",
	OnClick = function(self)
		ToggleDropDownMenu(nil, nil, dropdown, self, 0, 0)
	end,
})


local function onClick(self)
	if self:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
	end
	addon.db.options[self.setting] = self:GetChecked()
end

local function newOption(parent)
	local button = CreateFrame("CheckButton", nil, parent, "OptionsBaseCheckButtonTemplate")
	button:SetPushedTextOffset(0, 0)
	button:SetScript("OnClick", onClick)
	
	local text = button:CreateFontString(nil, nil, "GameFontHighlight")
	text:SetPoint("LEFT", button, "RIGHT", 0, 1)
	button:SetFontString(text)
	
	return button
end


local function setColor(id)
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local swatch = addon.swatches[id]
	swatch.tx:SetVertexColor(r, g, b)
	local color = swatch.color
	color.r = r
	color.g = g
	color.b = b
end

local function cancel(id, prev)
	local r, g, b = prev.r, prev.g, prev.b
	local swatch = addon.swatches[id]
	swatch.tx:SetVertexColor(r, g, b)
	local color = swatch.color
	color.r = r
	color.g = g
	color.b = b
end

local function onClick(self)
	local info = UIDropDownMenu_CreateInfo()
	local color = self.color
	info.r, info.g, info.b = color.r, color.g, color.b
	info.swatchFunc = self.swatchFunc
	info.cancelFunc = self.cancelFunc
	OpenColorPicker(info)
end

local function onEnter(self)
	self.bg:SetVertexColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
end

local function onLeave(self)
	self.bg:SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
end

local function newSwatch(parent)
	local swatch = CreateFrame("Button", nil, parent)
	swatch:SetSize(16, 16)
	swatch:SetPushedTextOffset(0, 0)
	
	swatch:SetNormalTexture([[Interface\ChatFrame\ChatFrameColorSwatch]])
	swatch.tx = swatch:GetNormalTexture()
	
	local bg = swatch:CreateTexture(nil, "BACKGROUND")
	bg:SetTexture(1.0, 1.0, 1.0)
	bg:SetSize(14, 14)
	bg:SetPoint("CENTER")
	swatch.bg = bg
	
	local text = swatch:CreateFontString(nil, nil, "GameFontHighlight")
	text:SetPoint("LEFT", swatch, "RIGHT", 5, 0)
	text:SetJustifyH("LEFT")
	swatch:SetFontString(text)
	
	swatch:SetScript("OnClick", onClick)
	swatch:SetScript("OnEnter", onEnter)
	swatch:SetScript("OnLeave", onLeave)
	
	return swatch
end


do
	local options = {
		{
			text = "Include current character",
			setting = "showPlayer",
		},
		{
			text = "Only same faction",
			setting = "sameFaction",
		},
		{
			text = "Show skill level",
			setting = "showSkillLevel",
		},
		{
			text = "Use side tooltip",
			setting = "separateTooltip",
		},
	}
	
	for i, v in ipairs(options) do
		local option = newOption(addon)
		if i == 1 then
			option:SetPoint("TOPLEFT", title, "BOTTOMLEFT", -2, -16)
		else
			option:SetPoint("TOP", options[i - 1], "BOTTOM", 0, -8)
		end
		option:SetText(v.text)
		option.setting = v.setting
		options[i] = option
	end
	
	addon.options = options
end


do
	local swatches = {
		{
			text = "Usable",
			setting = "usable",
		},
		{
			text = "Insufficient skill",
			setting = "lowSkill",
		},
		{
			text = "Wrong spec",
			setting = "noSpec",
		},
		{
			text = "Already known",
			setting = "knownBy",
		},
		{
			text = "Unuseful recipe",
			setting = "useless",
		},
	}
	
	for i, v in ipairs(swatches) do
		local swatch = newSwatch(addon)
		swatch:SetID(i)
		if i == 1 then
			swatch:SetPoint("TOP", addon.options[#addon.options], "BOTTOM", 0, -13)
		else
			swatch:SetPoint("TOP", swatches[i - 1], "BOTTOM", 0, -18)
		end
		swatch:SetText(v.text)
		swatch.setting = v.setting
		swatch.swatchFunc = function() setColor(i) end
		swatch.cancelFunc = function(prev) cancel(i, prev) end
		swatches[i] = swatch
	end
	
	addon.swatches = swatches
end


local colorRecipesMenu = {
	"bags",
	"bank",
	"guildBank",
	"vendor",
	"auction",
}

local colorRecipesMenuLabels = {
	bags = "Bags",
	bank = "Bank",
	guildBank = "Guild bank",
	vendor = "Vendors",
	auction = "Auction house",
}

local function onClick(self)
	addon.db.options.colorRecipes[self.value] = self.checked
	addon.colorRecipesFunc[self.value]()
end

local colorRecipes = CreateFrame("Frame", "BoCColorRecipesDropdown", addon, "UIDropDownMenuTemplate")
colorRecipes:SetPoint("TOPLEFT", addon.swatches[#addon.swatches], "BOTTOMLEFT", -16, -18)
colorRecipes.initialize = function(self)
	for i, v in ipairs(colorRecipesMenu) do
		local info = UIDropDownMenu_CreateInfo()
		info.text = colorRecipesMenuLabels[v]
		info.value = v
		info.func = onClick
		info.checked = addon.db.options.colorRecipes[v]
		info.isNotRadio = true
		info.keepShownOnClick = true
		UIDropDownMenu_AddButton(info)
	end
end
UIDropDownMenu_SetWidth(colorRecipes, 160)
UIDropDownMenu_SetText(colorRecipes, "Color recipes")
UIDropDownMenu_JustifyText(colorRecipes, "LEFT")


local popupData = {}

local function onClick(self, realm, name)
	local skill = self.value
	popupData.realm = realm
	popupData.name = name
	popupData.skill = skill
	skill = skills[skill]
	if skill then
		StaticPopup_Show("BOOKOFCRAFTS_RESET_SKILL", skill, name.." @ "..realm, popupData)
	else
		StaticPopup_Show("BOOKOFCRAFTS_RESET_CHARACTER", name, realm, popupData)
	end
end

local charDropdown = CreateFrame("Frame", "BoCUI_DropDownCharacters", addon, "UIDropDownMenuTemplate")
charDropdown:SetPoint("TOP", colorRecipes, "BOTTOM", 0, -8)
charDropdown.initialize = function(self, level)
	if level == 1 then
		for i, v in ipairs(addon.menu.realmNames) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.hasArrow = true
			info.notCheckable = true
			UIDropDownMenu_AddButton(info, level)
		end
	elseif level == 2 then
		local realm = UIDROPDOWNMENU_MENU_VALUE
		for i, v in ipairs(addon.menu[realm].charNames) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.value = realm.."|"..v
			info.func = onClick
			info.hasArrow = true
			info.notCheckable = true
			info.owner = self
			info.arg1 = realm
			info.arg2 = v
			UIDropDownMenu_AddButton(info, level)
		end
	elseif level == 3 then
		local realm, name = strsplit("|", UIDROPDOWNMENU_MENU_VALUE)
		for i, v in ipairs(addon.menu[realm].data[name]) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = skills[v]
			info.value = v
			info.func = onClick
			info.notCheckable = true
			info.owner = self
			info.arg1 = realm
			info.arg2 = name
			UIDropDownMenu_AddButton(info, level)
		end
	end
end
UIDropDownMenu_SetWidth(charDropdown, 160)
UIDropDownMenu_SetText(charDropdown, "Delete character data")
UIDropDownMenu_JustifyText(charDropdown, "LEFT")

StaticPopupDialogs["BOOKOFCRAFTS_RESET_CHARACTER"] = {
	text = "Are you sure you want to reset data for %s @ %s?",
	button1 = YES,
	button2 = NO,
	OnAccept = function(self, data)
		addon:DeleteCharacterData(data.realm, data.name)
		-- UIDropDownMenu_SetText(charDropdown, "Delete character data")
	end,
	OnCancel = function(self, data)
		-- UIDropDownMenu_SetText(charDropdown, "Delete character data")
	end,
	whileDead = true,
	timeout = 0,
}

StaticPopupDialogs["BOOKOFCRAFTS_RESET_SKILL"] = {
	text = "Are you sure you want to reset %s data for %s?",
	button1 = YES,
	button2 = NO,
	OnAccept = function(self, data)
		addon:DeleteCharacterData(data.realm, data.name)
	end,
	whileDead = true,
	timeout = 0,
}